"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const ink_1 = require("ink");
const path = require("path");
const util_1 = require("util");
const components_1 = require("./components");
const helpers_1 = require("./helpers");
class Container extends ink_1.Component {
    constructor(props) {
        super(props);
        this.state = {
            error: null,
            saved: false,
        };
        this.ensureDir = helpers_1.ensureDir;
        this.ensureFile = helpers_1.ensureFile;
    }
    get output() {
        return path.resolve(this.props.output);
    }
    ensureNotExistsFile(filename) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield util_1.promisify(fs.access)(filename, fs.constants.F_OK);
                throw new Error('Its file is already exists. Overwrite?');
            }
            catch (err) {
                if (err.message.includes('Overwrite?')) {
                    throw err;
                }
                else {
                    this.ensureFile(filename);
                }
            }
        });
    }
    componentDidUpdate() {
        if (this.state.saved) {
            process.exit(0);
        }
    }
    render() {
        return (ink_1.h(components_1.View, null,
            ink_1.h(components_1.Less, { content: this.props.content, save: () => __awaiter(this, void 0, void 0, function* () {
                    yield this.ensureDir(path.dirname(this.output));
                    try {
                        yield this.ensureNotExistsFile(this.output);
                    }
                    catch (err) {
                        if (this.state.error === null) {
                            this.setState({
                                error: err.message,
                            });
                            return;
                        }
                    }
                    fs.writeFileSync(this.output, this.props.content, 'utf-8');
                    this.setState({
                        saved: true,
                    });
                }) }),
            ink_1.h("div", null,
                ink_1.h("div", null),
                ink_1.h("div", null,
                    ink_1.h(ink_1.Text, { yellow: true }, "\u00BB"),
                    " Save? ",
                    ink_1.h(ink_1.Text, { blue: true }, "y"),
                    "/n",
                    ' ',
                    ink_1.h(ink_1.Text, { gray: true }, ":: down:j up:k")),
                this.state.error ? (ink_1.h("div", null,
                    ink_1.h(ink_1.Text, { red: true }, "!"),
                    " ",
                    this.state.error,
                    " ",
                    ink_1.h(ink_1.Text, { blue: true }, "y"),
                    "/n")) : null,
                this.state.saved ? (ink_1.h("div", null,
                    ink_1.h(ink_1.Text, { green: true }, "\u2714"),
                    " Created to ",
                    ink_1.h(ink_1.Text, { bold: true }, this.output))) : null)));
    }
}
exports.Container = Container;
//# sourceMappingURL=container.js.map