"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const matter = require("gray-matter");
const kebabCase = require("lodash.kebabcase");
const path = require("path");
const helpers_1 = require("./helpers");
class Context {
    static toKebabProps(obj, ignoreProps) {
        return Object.keys(obj).reduce(([data, propMap], originalKey) => {
            let key = originalKey;
            if (ignoreProps === undefined ||
                (ignoreProps !== undefined && ignoreProps.indexOf(key) === -1)) {
                key = kebabCase(originalKey);
            }
            propMap[key] = originalKey;
            data[key] = obj[originalKey];
            return [data, propMap];
        }, [{}, {}]);
    }
    constructor(filename, content, scope) {
        const ctx = helpers_1.formatMatter(matter(content));
        this.filename = filename;
        this.scope = scope;
        const [data, propMap] = Context.toKebabProps(ctx.data, [
            'CONFIG',
            'NOTE',
        ]);
        this.config = ctx.data.CONFIG;
        this.note = ctx.data.NOTE;
        this.data = data;
        this.propMap = propMap;
        this.content = ctx.content;
    }
    getOriginalProp(prop) {
        return this.propMap[prop];
    }
    forEachFlag(cb) {
        this.flags.forEach(flag => {
            cb(flag, this.data[flag]);
        });
    }
    getData(flag) {
        try {
            return this.config[flag] || false;
        }
        catch (_) {
            return false;
        }
    }
    get commandName() {
        return path.basename(this.filename, '.hbs');
    }
    get description() {
        return this.getData('description');
    }
    get usage() {
        return this.getData('usage');
    }
    get flags() {
        return Object.keys(this.data).filter(flag => {
            if (flag === 'CONFIG' || flag === 'NOTE') {
                return false;
            }
            return true;
        });
    }
}
exports.Context = Context;
//# sourceMappingURL=context.js.map