#!/usr/bin/env node
"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const chalk = require("chalk");
const Table = require("cli-table2");
const fs = require("fs");
const got = require("got");
const Handlebars = require("handlebars");
const ink_1 = require("ink");
const isUrl = require("is-url");
const compact = require("lodash.compact");
const path = require("path");
const util_1 = require("util");
const yargs = require("yargs");
const container_1 = require("./container");
const context_1 = require("./context");
const helpers_1 = require("./helpers");
process.on('unhandledRejection', (reason, p) => {
    // tslint:disable-next-line no-console
    console.error(reason);
});
(() => __awaiter(this, void 0, void 0, function* () {
    const filePatterns = compact([
        process.env.HOME !== undefined
            ? path.join(process.env.HOME, '.config/moza/*')
            : null,
        path.join(process.cwd(), '.moza/*'),
    ]);
    const ctxsAllPromises = filePatterns
        .map(filePattern => helpers_1.pglob(filePattern))
        .map((filenameSetPromise, idx) => {
        if (idx === 0) {
            return [filenameSetPromise, 'global'];
        }
        else {
            return [filenameSetPromise, 'local'];
        }
    })
        .map(([filenameSetPromise, scope]) => __awaiter(this, void 0, void 0, function* () {
        const filenameSet = yield filenameSetPromise;
        const ctxsSet = yield Promise.all(filenameSet.map((filename) => __awaiter(this, void 0, void 0, function* () {
            const content = yield util_1.promisify(fs.readFile)(filename, 'utf-8');
            return new context_1.Context(filename, content, scope);
        })));
        return ctxsSet;
    }))
        .reduce((accPromise, ctxsSetPromise) => __awaiter(this, void 0, void 0, function* () {
        const acc = yield accPromise;
        const ctxsSet = yield ctxsSetPromise;
        return [...acc, ...ctxsSet];
    }), []);
    if (Array.isArray(ctxsAllPromises)) {
        return;
    }
    const ctxs = yield ctxsAllPromises;
    ctxs.forEach((ctx) => {
        yargs.command(ctx.commandName, (() => {
            try {
                return ctx.description;
            }
            catch (_) {
                return '';
            }
        })(), command => {
            ctx.forEachFlag((flag, data) => {
                command.option(flag, data);
            });
            return command
                .group(ctx.flags, 'Variables:')
                .usage(ctx.usage || '')
                .help('help');
        }, (argv) => __awaiter(this, void 0, void 0, function* () {
            const [, destination] = argv._;
            if (destination === undefined) {
                // tslint:disable-next-line no-console
                console.error(chalk.red('Expected second arguments for destination path'));
                process.exit(1);
            }
            const flags = ctx.flags.reduce((acc, flag) => {
                const originalFlag = ctx.getOriginalProp(flag);
                acc[originalFlag] = argv[flag];
                if (acc[originalFlag] === null || acc[originalFlag] === undefined) {
                    acc[originalFlag] = ctx.data[flag].default;
                }
                return acc;
            }, {});
            require('./handlebars-helpers');
            const result = Handlebars.compile(ctx.content)(flags);
            ink_1.render(ink_1.h(container_1.Container, { content: result.trim(), flags: flags, output: path.resolve(destination) }));
        }));
    });
    yargs.command('ls', 'Show command list', command => {
        return command
            .option('global', {
            alias: 'g',
            default: false,
            type: 'boolean',
        })
            .usage('$ moza ls [-g]')
            .help();
    }, argv => {
        const table = new Table({
            head: ['Command', 'Path'],
        });
        ctxs.forEach((ctx) => {
            if (argv.global && ctx.scope === 'global') {
                /**
                 * 型がおかしいので`any`化
                 */
                table.push([chalk.yellow(ctx.commandName), ctx.filename]);
            }
            else if (!argv.global && ctx.scope === 'local') {
                /**
                 * 同上
                 */
                table.push([chalk.yellow(ctx.commandName), ctx.filename]);
            }
        });
        // tslint:disable-next-line no-console
        console.log(table.toString());
        process.exit(0);
    });
    yargs.command('note', 'Show command note', command => {
        return command.usage('$ moza note <command>').help();
    }, argv => {
        const [, targetName] = argv._;
        if (!targetName) {
            // tslint:disable-next-line no-console
            console.error(chalk.red('Specify command name for first argument'));
            process.exit(1);
        }
        const targetCtx = ctxs.find(ctx => ctx.commandName === targetName);
        if (targetCtx === undefined) {
            // tslint:disable-next-line no-console
            console.error(chalk.red('The specified command name does not exist'));
            process.exit(1);
        }
        else {
            // tslint:disable-next-line no-console
            console.log(`
${chalk.yellow('##')} ${chalk.bold(targetCtx.commandName)}

${targetCtx.note}`);
        }
    });
    yargs.command('add', 'Fetch a template', command => {
        return command
            .option('global', {
            alias: 'g',
            default: false,
            type: 'boolean',
        })
            .option('output', {
            alias: 'o',
            demandOption: true,
            type: 'string',
        })
            .usage('$ yarn add [-g] <url>')
            .help();
    }, (argv) => __awaiter(this, void 0, void 0, function* () {
        const [, url] = argv._;
        if (!isUrl(url)) {
            // tslint:disable-next-line no-console
            console.error(chalk.red('Specify url for first argument'));
            process.exit(1);
        }
        const content = yield (() => __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield got(url);
                return response.body;
            }
            catch (err) {
                // tslint:disable-next-line no-console
                console.log(chalk.red(err.message));
                process.exit(1);
            }
        }))();
        if (argv.global) {
            const dirname = path.resolve(process.env.HOME, '.config/moza');
            const filename = path.join(dirname, argv.output);
            yield helpers_1.ensureDir(dirname);
            yield helpers_1.ensureFile(filename);
            yield util_1.promisify(fs.writeFile)(filename, content);
        }
        else {
            const dirname = path.resolve(process.cwd(), '.moza');
            const filename = path.join(dirname, argv.output);
            yield helpers_1.ensureDir(dirname);
            yield helpers_1.ensureFile(filename);
            yield util_1.promisify(fs.writeFile)(filename, content);
        }
    }));
    return yargs.argv;
}))();
//# sourceMappingURL=moza.js.map